#==============================================================================
# Widget zur komfortablen Datums- und Zeitangabe ala Win95-Systemsteuerung
#==============================================================================
# Copr (c) 1997-98 ComConsult Kommunikationstechnik GmbH
# Autor: Richard Breuer
#==============================================================================
# 25.01.98 RB	Namespace definiert, Bugfix fuer einstellige Monate, getDate
#		akzeptiert ein Defaultdatum
# 08.12.97 RB   Bugfix: Tage mit fuehrender 0 waren fehlerhaft
# 28.10.97 RB   Erste Version
#==============================================================================
# BEISPIELE:
#	source date.tcl
#	namespace import TkDate::*
#	set d [getDate]				--> 25.01.1998 22:57
#	set d [getDate "01.01.1998 04:35"]	--> 01.01.1998 04:35
#	set d [getDate "01.01.1998 04:35" 2]	--> 01.01.98 04:35
#	set d [getDate "" 2]			--> 25.01.98 22:58
#==============================================================================
# CREDITS:
#	Die Grafikteile wurden zum groessten Teil mit Stewart Allens tollem
#	Programm VisualTcl erstellt. Download unter http://www.neuron.com
#==============================================================================

namespace eval TkDate {

  namespace export getDate

  variable date

  set date(year) {}
  set date(hhmm) {}
  set date(monthname) {}

  set date(widget,dateWindow) {.topdate}
  set date(widget,calendar) {.topdate.fra21.01.02.09.fra23}
  set date(widget,month) {.topdate.fra21.01.02.03.04}

  proc dateWindow {args} {
    set base .topdate
    if {[winfo exists .topdate]} {
        wm deiconify .topdate; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel .topdate -class Toplevel \
        -cursor xterm 
    wm focusmodel .topdate passive
    wm geometry .topdate 233x337
    wm maxsize .topdate 233 337
    wm minsize .topdate 233 337
    wm overrideredirect .topdate 0
    wm resizable .topdate 1 1
    wm deiconify .topdate
    wm title .topdate "Datum/Uhrzeit"
    frame .topdate.fra21 \
        -borderwidth 1 -height 30 -width 30 
    frame .topdate.fra21.01 \
        -borderwidth 1 -height 30 -width 30 
    frame .topdate.fra21.01.02 \
        -borderwidth 1 -height 30 -relief sunken -width 30 
    frame .topdate.fra21.01.02.03 \
        -borderwidth 1 -height 30 -width 30 
    menubutton .topdate.fra21.01.02.03.04 \
        -background #c7c7c7 -menu .topdate.fra21.01.02.03.04.05 \
	-padx 5 -pady 4 \
        -relief sunken -textvariable TkDate::date(monthname) 
    menu .topdate.fra21.01.02.03.04.05
    button .topdate.fra21.01.02.03.06 \
        -command TkDate::incrYear -padx 2 -pady 2 -text + 
    entry .topdate.fra21.01.02.03.07 \
        -background #c7c7c7 -state disabled \
	-textvariable TkDate::date(year) -width 5 
    button .topdate.fra21.01.02.03.08 \
        -command TkDate::decrYear -padx 2 -pady 2 -text - 
    frame .topdate.fra21.01.02.09 \
        -borderwidth 1 -height 30 -width 30 
    frame .topdate.fra21.01.02.09.fra23 \
        -borderwidth 1 -height 30 -relief sunken -width 30 
    frame .topdate.fra21.01.010 \
        -borderwidth 1 -height 30 -relief raised -width 30 
    label .topdate.fra21.01.010.011 \
        -anchor w -relief groove -text Uhrzeit 
    entry .topdate.fra21.01.010.012 \
        -cursor {} -highlightthickness 0 \
	-textvariable TkDate::date(hhmm) -width 6 
    frame .topdate.fra22 \
        -borderwidth 1 -height 30 -relief sunken -width 30 
    button .topdate.fra22.but39 \
        -command {destroy $TkDate::date(widget,dateWindow)} \
	-padx 11 -pady 4 \
        -text Ok 
    button .topdate.fra22.but40 \
        -command {set TkDate::date(year) ""
destroy $TkDate::date(widget,dateWindow)} \
        -padx 11 -pady 4 -text Abbrechen 
    button .topdate.fra22.but41 \
        -padx 11 -pady 4 -text Hilfe 
    ###################
    # SETTING GEOMETRY
    ###################
    pack .topdate.fra21 \
        -anchor center -expand 1 -fill both -side top 
    pack .topdate.fra21.01 \
        -anchor center -expand 1 -fill both -side top 
    pack .topdate.fra21.01.02 \
        -anchor center -expand 1 -fill both -side top 
    pack .topdate.fra21.01.02.03 \
        -anchor center -expand 0 -fill x -side top 
    pack .topdate.fra21.01.02.03.04 \
        -anchor center -expand 0 -fill none -padx 10 -pady 5 -side left 
    pack .topdate.fra21.01.02.03.06 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side right 
    pack .topdate.fra21.01.02.03.07 \
        -anchor center -expand 0 -fill none -pady 5 -side right 
    pack .topdate.fra21.01.02.03.08 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side right 
    pack .topdate.fra21.01.02.09 \
        -anchor center -expand 1 -fill both -side top 
    pack .topdate.fra21.01.02.09.fra23 \
        -anchor center -expand 1 -fill both -padx 10 -pady 10 -side top 
    pack .topdate.fra21.01.010 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side top 
    pack .topdate.fra21.01.010.011 \
        -anchor center -expand 0 -fill none -padx 2 -pady 2 -side left 
    pack .topdate.fra21.01.010.012 \
        -anchor center -expand 1 -fill x -padx 2 -pady 2 -side right 
    pack .topdate.fra22 \
        -anchor center -expand 0 -fill x -side top 
    pack .topdate.fra22.but39 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side left 
    pack .topdate.fra22.but40 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side left 
    pack .topdate.fra22.but41 \
        -anchor center -expand 0 -fill none -padx 5 -pady 5 -side right 
}

set date(DAYS,1) {31}
set date(DAYS,2) {28}
set date(DAYS,3) {31}
set date(DAYS,4) {30}
set date(DAYS,5) {31}
set date(DAYS,6) {30}
set date(DAYS,7) {31}
set date(DAYS,8) {31}
set date(DAYS,9) {30}
set date(DAYS,10) {31}
set date(DAYS,11) {30}
set date(DAYS,12) {31}

set date(MONTHNAMES) {Januar Februar Maerz April Mai Juni Juli\
                      August September Oktober November Dezember}

# Beschraenkung fuer zweistellige Jahreszahlen ala "clock scan"
set date(MINYEAR) 1969
set date(MAXYEAR) 2038

set date(IX,Mon) 0
set date(IX,Tue) 1
set date(IX,Wed) 2
set date(IX,Thu) 3
set date(IX,Fri) 4
set date(IX,Sat) 5
set date(IX,Sun) 6

proc dow {d m y} {
  variable date
  set date(dow) \
    [clock format [clock scan "$m/$d/$y"] -format %a]
}

proc month2name {m} {
  variable date
  return [lindex $date(MONTHNAMES) [expr $m-1]]
}

proc name2month {monthname} {
  variable date
  return [expr 1+[lsearch -exact $date(MONTHNAMES) $monthname]]
}

proc daysInMonth {m y} {
  variable date
  if {$date(month)==2 && [leap]} {return 29} \
  else {return $date(DAYS,$date(month))}
}

proc decrYear {} {
  variable date
  if {$date(year)>$date(MINYEAR)} {
    incr date(year) -1
    if {$date(month)==2 && $date(day)>28} {set date(day) 28}
    dow $date(day) $date(month) $date(year)
    buildCalendar
  }
}

proc incrYear {} {
  variable date
  if {$date(year)<$date(MAXYEAR)} {
    incr date(year)
    if {$date(month)==2 && $date(day)>28} {set date(day) 28}
    dow $date(day) $date(month) $date(year)
    buildCalendar
  }
}

proc setMonth {monthname} {
  variable date
  set date(month) [name2month $monthname]
  set date(monthname) $monthname
  if {$date(month)==2 && $date(day)>28 &&![leap]} {set date(day) 28}
  buildCalendar
}

proc leap {} {
  variable date
  if {[expr $date(year)%400]==0} {return 1}
  if {[expr $date(year)%100]==0} {return 0}
  if {[expr $date(year)%4]==0} {return 1}
  return 0
}

proc getDate {{defDateTime ""} {yearDigits 4}} {
  variable date

  if {$defDateTime==""} {
    scan [clock format [clock seconds] -format %d] %d date(day)
    scan [clock format [clock seconds] -format %m] %d date(month)
    scan [clock format [clock seconds] -format %Y] %d date(year)
    set date(hhmm) [clock format [clock seconds] -format %H:%M]
  } else {
    set blankPos [string last " " $defDateTime]
    if {$blankPos<0} {
      set date(hhmm) [clock format [clock seconds] -format %H:%M]
    } else {
      set date(hhmm) [string range $defDateTime [expr 1+$blankPos] end]
    }

    # String nach Ami-Format umformen
    set defDateTime "[string range $defDateTime 3 4]/[string range $defDateTime 0 1]/[string range $defDateTime 6 end]"
    if {[catch {set s [clock scan $defDateTime]}]} {
      set s [clock seconds]
    }
    scan [clock format $s -format %d] %d date(day)
    scan [clock format $s -format %m] %d date(month)
    scan [clock format $s -format %Y] %d date(year)

    if {$date(year)<$date(MINYEAR)} {set date(year) $date(MINYEAR)}
    if {$date(year)>$date(MAXYEAR)} {set date(year) $date(MAXYEAR)}
  }

  set date(monthname) [month2name $date(month)]
  dow $date(day) $date(month) $date(year)
  dateWindow
  buildMenu $date(widget,month) $date(MONTHNAMES) TkDate::setMonth
  buildCalendar
  raise $date(widget,dateWindow)
  grab $date(widget,dateWindow)
  tkwait window $date(widget,dateWindow)

  if {$date(year)==""} {return ""}

  if {[scan $date(hhmm) "%d:%d" hh mm]!=2} {
    set hh 0
    set mm 0
  }
  set format "%02d.%02d.%0"
  append format $yearDigits
  append format "d %02d:%02d"
  if {$yearDigits==2} {set date(year) [expr $date(year) % 100]}
  #puts "format $format $date(day) $date(month) $date(year) $hh $mm"
  return [format $format $date(day) $date(month) $date(year) $hh $mm]
}

proc buildMenu {menu list {cmd ccmTrace}} {
  # Erzeuge die Menueeintraege in $menu.m aus den Eintraegen in $list. $cmd
  # gibt an, welches Kommando auszufuehren ist, wenn der Menueeintrag gewaehlt
  # wird
  catch {destroy [set menu].m}
  menu [set menu].m -tearoff 0
  if {$list=={}} {
    $menu configure -state disabled -relief flat
  } else {
    $menu configure -state normal -relief sunken
    foreach e $list {
      [set menu].m add command -label $e -command "$cmd \"$e\""
    }
  }
  $menu configure -menu [set menu].m
}

proc moveDay {to} {
  variable date

  set w $date(widget,calendar)
  set from $date(day)

  # demarkiere den alten und markiere den neuen Knopf
  $w.$from configure -background white -foreground black
  $w.$to   configure -background blue  -foreground white

  # Setze den aktuellen Tag
  set date(day) $to
}

proc buildCalendar {} {
  variable date

  set w $date(widget,calendar)
  catch {eval destroy [winfo children $w]}

  # Header erzeugen
  set startCol 0
  foreach d {Mo Di Mi Do Fr Sa So} {
    set dd [string tolower $d]
    label $w.$dd -relief groove -text $d -background #c9c9c9
    grid $w.$dd -column $startCol -row 0 -sticky ew
    incr startCol
  }

  # Schleife ueber die Tage
  set days [daysInMonth $date(month) $date(year)]
  set startCol $date(IX,[dow 1 $date(month) $date(year)])
  set startRow 1
  for {set i 1} {$i<=$days} {incr i} {
    button $w.$i -relief flat -text $i -background white -foreground black\
      -command "TkDate::moveDay $i" -padx 2 -pady 2
    if {$date(day)==$i} {
      $w.$i configure -background blue -foreground white
    }
    grid $w.$i -column $startCol -row $startRow -sticky ew
    if {$startCol==6} {set startCol 0; incr startRow} else {incr startCol}
  }
}

}
